#    SOLDESA es un desarrollo del Departamento de Evaluacion Sensorial de
#    Alimentos del Instituto Superior Experimental de Tecnologia Alimentaria
#    (DESA-ISETA) y de la Comision de Investigaciones Cientificas de la provincia
#    de Buenos Aires (CIC), con la colaboracion de la Cooperativa de Software 
#    Libre ELVEX
#    Mail: graciela@desa.edu.ar
#    Sitio Web: www.desa.edu.ar    
#
#    Copyright (c) 2013 DESA/ISETA   
#    This file is part of Soldesa.
#
#    Soldesa is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Soldesa is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Soldesa.  If not, see <http://www.gnu.org/licenses/>.
import datetime
import hashlib
import uuid
import copy

from flask import g
from flaskext.couchdb import Document, TextField, DateTimeField, ListField, \
           DictField, FloatField, BooleanField, IntegerField, IntegerField, ViewField

def clone(doc, set_uuid=True):
    ''' Clones a Document '''
    data = copy.deepcopy(doc.unwrap())
    del(data['_id'])
    del(data['_rev'])
    new_doc = type(doc).wrap(data)
    if set_uuid:
        new_doc.id = str(uuid.uuid1()).replace('-', '')
    return new_doc

class User(Document):
    doc_type = "user"
    username = TextField()
    password = TextField()
    group = TextField()
    roles = ListField(TextField())

    view_all = ViewField('user_all', '''\
        function (doc) {
            if (doc.doc_type == 'user') {
                  emit(doc.username, doc);
            };
        }''')
    view_role = ViewField('user_role', '''\
        function (doc) {
            if (doc.doc_type == 'user') {
                  for(role in doc.roles){
                    emit(doc.roles[role], doc);
                  }
            };
        }''')

    view_group = ViewField('user_group', '''\
        function (doc) {
            if (doc.doc_type == 'user' && doc.group) {
                  emit(doc.group, null);
            };
        }''','''
        function(keys, values, reduce) {
            return null;
        }'''
        )

    def set_password(self, password):
        from couchie import app
        secret = app.config['SECRET']
        hexhash = hashlib.sha512(secret+password).hexdigest()
        self.password = hexhash

    def check_password(self, password):
        from couchie import app
        secret = app.config['SECRET']
        hexhash = hashlib.sha512(secret+password).hexdigest()
        return self.password == hexhash

class Panel(Document):
    doc_type = "panel"
    nombre = TextField()
    fecha = DateTimeField(default=datetime.datetime.now)
    lider = TextField()
    tipo = TextField()
    activo = BooleanField(default=False)
    finished = BooleanField(default=False)
    archived = BooleanField(default=False)
    atributos = ListField(TextField())
    min_escala = FloatField()
    max_escala = FloatField()
    ancla = FloatField()

    contributors = ListField(TextField())

    def clone(self):
        new_doc = clone(self)
        # TODO: use include_docs
        attrs = [clone(Atributo.load(attr)) for attr in self.atributos]
        new_doc.atributos = [attr.id for attr in attrs]
        # bulk save
        g.couch.update([new_doc] + attrs)
        return Panel.load(new_doc.id)

    view_all = ViewField('panel_all', '''\
        function (doc) {
            if (doc.doc_type == 'panel' && doc.archived != true) {
                  emit(doc.nombre, doc);
            };
        }''')

    view_by_lider = ViewField('panel_all', '''\
        function (doc) {
            if (doc.doc_type == 'panel' && doc.archived != true) {
                  emit(doc.lider, doc);
            };
        }''')

    view_archived = ViewField('panel_all', '''\
        function (doc) {
            if (doc.doc_type == 'panel' && doc.archived == true) {
                  emit(doc.nombre, doc);
            };
        }''')


class Atributo(Document):
    # TODO: change desciptor to atributo
    doc_type = "descriptor"
    nombre = TextField()
    descriptores = DictField()
    anclas = DictField()
    session = TextField()
    rotulo = TextField()

    view_all = ViewField('descriptor_all', '''\
        function (doc) {
            if (doc.doc_type == 'descriptor') {
                  emit(doc.nombre, doc);
            };
        }''')

    view_descriptores = ViewField('atributo_descriptores', '''\
        function (doc) {
           if(doc.doc_type == 'descriptor' && doc.descriptores) {
               for(desc in doc.descriptores){
                 emit([doc.nombre.toLowerCase(), desc], null);
               }
           }
        }''',
        '''function(keys, values, reduce) {
            return null;
        }''', group=True, reduce=True)

    view_base = ViewField('descriptor_base', '''\
        function (doc) {
            if (doc.doc_type == 'descriptor' && doc.session == null) {
                  emit(doc.id, doc);
            };
        }''')

    view_session = ViewField('descriptor_session', '''\
        function (doc) {
            if (doc.doc_type == 'descriptor' && doc.session != null) {
                  emit(doc.session, doc);
            };
        }''')


class Session(Document):
    doc_type = "session"
    panel_id = TextField()
    session_id = IntegerField()
    fecha = DateTimeField(default=datetime.datetime.now)
    repeticion = IntegerField()
    tope = IntegerField(default=10)
    evaluador = TextField()
    id_evaluador = TextField()
    muestra = IntegerField() 
    rotulo = TextField()
    vasito = IntegerField()
    orden = IntegerField()
    atributos = DictField(default={})
    closed = BooleanField(default=False)
    comment = TextField()

    view_all = ViewField('session_all', '''\
        function (doc) {
            if (doc.doc_type == 'session') {
                  emit(doc.nombre, doc);
            };
        }''')

    view_panel_closed = ViewField('session_panel_closed', '''\
         function(doc) {
           if(doc.doc_type == 'session'){
              emit([doc.panel_id, doc.closed], null);
            }
         }''',
        '''function(keys, values, reduce) {
            return null;
        }''')

    view_evaluador = ViewField('session_evaluador', '''\
        function (doc) {
            if (doc.doc_type == 'session') {
                  emit([doc.panel_id, doc.evaluador, doc.id_evaluador], doc);
            };
        }''','''
        function(keys, values, reduce) {
            return null;
        }'''
        )

    view_evaluador_panel = ViewField('session_evaluador_panel', '''\
        function (doc) {
            if (doc.doc_type == 'session') {
                  emit([doc.id_evaluador, doc.panel_id], doc);
            };
        }''','''
        function(keys, values, reduce) {
            return null;
        }'''
        )

    view_user_sessions = ViewField('session_panel_rep', '''\
        function (doc) {
            if (doc.doc_type == 'session') {
                  emit([doc.id_evaluador, doc.panel_id, doc.session_id], doc);
            };
        }''','''
        function(keys, values, reduce) {
            return null;
        }'''
        )

    view_rep = ViewField('session_panel_rep', '''\
        function (doc) {
            if (doc.doc_type == 'session') {
                  emit([doc.panel_id, doc.repeticion], doc);
            };
        }''','''
        function(keys, values, reduce) {
            return null;
        }''', reduce=True, group=True
        )

    view_muestras = ViewField('session_muestras', '''\
        function (doc) {
            if (doc.doc_type == 'session') {
                  emit([doc.panel_id, doc.muestra, doc.rotulo], doc);
            };
        }''','''
        function(keys, values, reduce) {
            return null;
        }'''
        )


    view_rotulos = ViewField('session_rotulos', '''\
        function (doc) {
          if (doc.doc_type == 'session') {
            emit([doc.panel_id, doc.rotulo], null);
          };
        }
        ''', '''\
        function(keys, values, reduce) {
            return {};
        }''', reduce=True, group=True
        )

    view_panel_ids = ViewField('session_panels_ids', '''\
        function (doc) {
          if (doc.doc_type == 'session') {
            emit([doc.panel_id, doc.session_id], doc);
          };
        }
        ''', '''\
        function(keys, values, reduce) {
            return {};
        }''', reduce=False
        )

    view_muestras_session = ViewField('session_panels', '''\
        function (doc) {
          if (doc.doc_type == 'session') {
            emit([doc.panel_id, doc.session_id,  doc.muestra], doc);
          };
        }
        ''', '''\
        function(keys, values, reduce) {
            return {};
        }''', reduce=True, group=True
        )

    view_muestras_session_user = ViewField('session_panels', '''\
        function (doc) {
          if (doc.doc_type == 'session') {
            emit([doc.id_evaluador, doc.panel_id, doc.session_id,  doc.muestra], doc);
          };
        }
        '''
        )
